/*! \file
 * \brief Howtouse, Sorter.h
 *
 * Test with: 
 * g++ -Wall Sorter-example.C -o Sorter-example
 *
 */

#include <iostream>
#include <vector>
#include <list>
#include <iterator>
#include "Sorter.h"

using namespace std;

// compares 2 integers
struct cmpInts
{
	bool operator () (int i1, int i2) { return i1<i2;}
};

int main()
{
	int data[] = {1,3,9,1,-4,7,2};								//will be sorted by sort_elements
	vector<int> vData1(data, data+sizeof(data)/sizeof(int)); 	//will be sorted by std::sort
	vector<int> vData2 = vData1;							 	//will be sorted by sort_elements
	list<int> 	lData(data, data+sizeof(data)/sizeof(int));		//will be sorted by sort_elements

	cout << "Original data\n";
	copy(vData1.begin(), vData1.end(), ostream_iterator<int>(cout, "\n"));
	
	sort(vData1.begin(), vData1.end(), cmpInts());
	cout << "Sort with std::sort\n";
	copy(vData1.begin(), vData1.end(), ostream_iterator<int>(cout, "\n"));

	cout << "Sort with sort_elements, with vector\n";
	typedef vector<int>::iterator T;
	vector<SorterElement<T> > res = sort_elements(vData2.begin(), vData2.end(),cmpInts());
	for (vector<SorterElement<T> >::iterator i=res.begin(); i!=res.end(); ++i)
		cout << i->_ind << ":" << (*i->_e) << "\n";

	cout << "Sort with sort_elements, with carray\n";
	typedef int * T2;
	vector<SorterElement<T2> > res2 = sort_elements(data, data+sizeof(data)/sizeof(int), cmpInts());
	for (vector<SorterElement<T2> >::iterator i=res2.begin(); i!=res2.end(); ++i)
		cout << i->_ind << ":" << (*i->_e) << "\n";

	cout << "Sort with sort_elements, with list\n";
	typedef list<int>::iterator T3;
	vector<SorterElement<T3> > res3 = sort_elements(lData.begin(), lData.end(),cmpInts());
	for (vector<SorterElement<T3> >::iterator i=res3.begin(); i!=res3.end(); ++i)
		cout << i->_ind << ":" << (*i->_e) << "\n";
	
	cout << "CCL = OK if same order as std::sort, and give correct position in original container\n";

	return 0;
}

